//------------------------------------------------------------------
//
//------------------------------------------------------------------
#property indicator_separate_window
#property indicator_buffers 3
#property indicator_color1  LimeGreen
#property indicator_color2  OrangeRed
#property indicator_color3  OrangeRed
#property indicator_width1  2
#property indicator_width2  2
#property indicator_width3  2

//
//
//
//
//

extern string TimeFrame            = "Current time frame";
extern int    Mom_Period           = 1;
extern int    T3_Period            = 28;
extern bool   T3_original          = false;
extern double b                    = 0.618;
extern double levelUp              = 0.003;
extern double levelDn              = -0.003;
extern bool   verticalLinesVisible = true;
extern string verticalLinesID      = "T3BurstLines";
extern color  verticalLinesUpColor = LimeGreen;
extern color  verticalLinesDnColor = Orange;
extern int    verticalLinesStyle   = STYLE_DOT;
extern int    verticalLinesWidth   = 0;
extern bool   Interpolate          = true;

//
//
//
//
//

double MapBuffer[];
double MapBufferda[];
double MapBufferdb[];
double upstate[];
double dnstate[];
double zestate[];

string indicatorFileName;
bool   returnBars;
bool   calculateValue;
int    timeFrame;

//------------------------------------------------------------------
//
//------------------------------------------------------------------
//
//
//
//
//

int init()
{
    IndicatorBuffers(6);
    SetIndexBuffer(0,MapBuffer);
    SetIndexBuffer(1,MapBufferda);
    SetIndexBuffer(2,MapBufferdb);
    SetIndexBuffer(3,upstate);
    SetIndexBuffer(4,dnstate);
    SetIndexBuffer(5,zestate);
    
      //
      //
      //
      //
      //
      
      indicatorFileName = WindowExpertName();
      calculateValue    = TimeFrame=="calculateValue"; if (calculateValue) { return(0); }
      returnBars        = TimeFrame=="returnBars";     if (returnBars)     { return(0); }
      timeFrame         = stringToTimeFrame(TimeFrame);
      SetLevelValue(0,levelUp);
      SetLevelValue(1,levelDn);
      SetLevelValue(2,0);
      
      //
      //
      //
      //
      //
      
    IndicatorShortName(timeFrameToString(timeFrame)+" T3_Momentum_Burst "+T3_Period);
    return(0);
}
int deinit()
{
   string lookFor       = verticalLinesID+":";
   int    lookForLength = StringLen(lookFor);
   for (int i=ObjectsTotal()-1; i>=0; i--)
   {
      string objectName = ObjectName(i);
         if (StringSubstr(objectName,0,lookForLength) == lookFor) ObjectDelete(objectName);
   }
   return(0);
}


//------------------------------------------------------------------
//
//------------------------------------------------------------------
int start()
{
   int counted_bars=IndicatorCounted();
   int i,limit;

   if(counted_bars<0) return(-1);
   if(counted_bars>0) counted_bars--;
         limit = MathMin(Bars-counted_bars,Bars-1);
         if (returnBars) { MapBuffer[0] = limit+1; return(0); }

   //
   //
   //
   //
   //

   if (calculateValue || timeFrame==Period())
   {
     if (zestate[limit]==-1) CleanPoint(limit,MapBufferda,MapBufferdb);
     for(i=limit; i>=0; i--)
     {
        MapBuffer[i]   = iT3(iMomentum(NULL,0,Mom_Period,PRICE_CLOSE,i)-iMomentum(NULL,0,Mom_Period,PRICE_CLOSE,i+1),T3_Period,b,T3_original,i);
        MapBufferda[i] = EMPTY_VALUE;
        MapBufferdb[i] = EMPTY_VALUE;
        upstate[i] = upstate[i+1];
        dnstate[i] = dnstate[i+1];
        zestate[i] = zestate[i+1];
          if (MapBuffer[i]>levelUp) upstate[i] =  1;
          if (MapBuffer[i]<levelUp) upstate[i] = -1;
          if (MapBuffer[i]>levelDn) dnstate[i] =  1;
          if (MapBuffer[i]<levelDn) dnstate[i] = -1;
          if (MapBuffer[i]>0)       zestate[i] =  1;
          if (MapBuffer[i]<0)       zestate[i] = -1;
          if (zestate[i]==-1) PlotPoint(i,MapBufferda,MapBufferdb,MapBuffer);                  
          if (verticalLinesVisible)
          {
             ObjectDelete(verticalLinesID+":u:"+Time[i]);
             ObjectDelete(verticalLinesID+":d:"+Time[i]);
             if (upstate[i] != upstate[i+1] && upstate[i]== 1) drawLine(":u:",i,verticalLinesUpColor);
             if (dnstate[i] != dnstate[i+1] && dnstate[i]==-1) drawLine(":d:",i,verticalLinesDnColor);
          }            
     }      
     return(0);
   }
     
   //
   //
   //
   //
   //
   
   limit = MathMax(limit,MathMin(Bars-1,iCustom(NULL,timeFrame,indicatorFileName,"returnBars",0,0)*timeFrame/Period()));
   if (zestate[limit]==-1) CleanPoint(limit,MapBufferda,MapBufferdb);
   for (i=limit; i>=0; i--)
   {
      int y = iBarShift(NULL,timeFrame,Time[i]);
         MapBufferda[i] = EMPTY_VALUE;
         MapBufferdb[i] = EMPTY_VALUE;
         MapBuffer[i]   = iCustom(NULL,timeFrame,indicatorFileName,"calculateValue",Mom_Period,T3_Period,T3_original,b,levelUp,levelDn,verticalLinesVisible,verticalLinesID,verticalLinesUpColor,verticalLinesDnColor,verticalLinesStyle,verticalLinesWidth,0,y);
         zestate[i]     = iCustom(NULL,timeFrame,indicatorFileName,"calculateValue",Mom_Period,T3_Period,T3_original,b,levelUp,levelDn,verticalLinesVisible,verticalLinesID,verticalLinesUpColor,verticalLinesDnColor,verticalLinesStyle,verticalLinesWidth,5,y);

         //
         //
         //
         //
         //
      
         if (!Interpolate || y==iBarShift(NULL,timeFrame,Time[i-1])) continue;

         //
         //
         //
         //
         //

         datetime time = iTime(NULL,timeFrame,y);
            for(int n = 1; i+n < Bars && Time[i+n] >= time; n++) continue;	
            for(int k = 1; k < n; k++)
            {
               MapBuffer[i+k] = MapBuffer[i] + (MapBuffer[i+n] - MapBuffer[i])*k/n;
            }
                                
   }
   for (i=limit; i>=0; i--) if (zestate[i]==-1) PlotPoint(i,MapBufferda,MapBufferdb,MapBuffer);  
return(0);
}

//------------------------------------------------------------------
//
//------------------------------------------------------------------
//
//
//
//
//

void drawLine(string add, int i,color theColor)
{
   string name = verticalLinesID+add+Time[i];
   
      //
      //
      //
      //
      //
      
      ObjectCreate(name,OBJ_VLINE,0,Time[i],0);
         ObjectSet(name,OBJPROP_COLOR,theColor);
         ObjectSet(name,OBJPROP_STYLE,verticalLinesStyle);
         ObjectSet(name,OBJPROP_WIDTH,verticalLinesWidth);
         ObjectSet(name,OBJPROP_BACK,true);
}

//------------------------------------------------------------------
//
//------------------------------------------------------------------
//
//
//
//
//

double workT3[][6];
double workT3Coeffs[][6];
#define _period 0
#define _c1     1
#define _c2     2
#define _c3     3
#define _c4     4
#define _alpha  5

//
//
//
//
//

double iT3(double price, double period, double hot, bool original, int i, int instanceNo=0)
{
   if (ArrayRange(workT3,0) != Bars)                ArrayResize(workT3,Bars);
   if (ArrayRange(workT3Coeffs,0) < (instanceNo+1)) ArrayResize(workT3Coeffs,instanceNo+1);

   if (workT3Coeffs[instanceNo][_period] != period)
   {
     workT3Coeffs[instanceNo][_period] = period;
        double a = hot;
            workT3Coeffs[instanceNo][_c1] = -a*a*a;
            workT3Coeffs[instanceNo][_c2] = 3*a*a+3*a*a*a;
            workT3Coeffs[instanceNo][_c3] = -6*a*a-3*a-3*a*a*a;
            workT3Coeffs[instanceNo][_c4] = 1+3*a+a*a*a+3*a*a;
            if (original)
                 workT3Coeffs[instanceNo][_alpha] = 2.0/(1.0 + period);
            else workT3Coeffs[instanceNo][_alpha] = 2.0/(2.0 + (period-1.0)/2.0);
   }
   
   //
   //
   //
   //
   //
   
   int buffer = instanceNo*6;
   int r = Bars-i-1;
   if (r == 0)
      {
         workT3[r][0+buffer] = price;
         workT3[r][1+buffer] = price;
         workT3[r][2+buffer] = price;
         workT3[r][3+buffer] = price;
         workT3[r][4+buffer] = price;
         workT3[r][5+buffer] = price;
      }
   else
      {
         workT3[r][0+buffer] = workT3[r-1][0+buffer]+workT3Coeffs[instanceNo][_alpha]*(price              -workT3[r-1][0+buffer]);
         workT3[r][1+buffer] = workT3[r-1][1+buffer]+workT3Coeffs[instanceNo][_alpha]*(workT3[r][0+buffer]-workT3[r-1][1+buffer]);
         workT3[r][2+buffer] = workT3[r-1][2+buffer]+workT3Coeffs[instanceNo][_alpha]*(workT3[r][1+buffer]-workT3[r-1][2+buffer]);
         workT3[r][3+buffer] = workT3[r-1][3+buffer]+workT3Coeffs[instanceNo][_alpha]*(workT3[r][2+buffer]-workT3[r-1][3+buffer]);
         workT3[r][4+buffer] = workT3[r-1][4+buffer]+workT3Coeffs[instanceNo][_alpha]*(workT3[r][3+buffer]-workT3[r-1][4+buffer]);
         workT3[r][5+buffer] = workT3[r-1][5+buffer]+workT3Coeffs[instanceNo][_alpha]*(workT3[r][4+buffer]-workT3[r-1][5+buffer]);
      }

   //
   //
   //
   //
   //
   
   return(workT3Coeffs[instanceNo][_c1]*workT3[r][5+buffer] + 
          workT3Coeffs[instanceNo][_c2]*workT3[r][4+buffer] + 
          workT3Coeffs[instanceNo][_c3]*workT3[r][3+buffer] + 
          workT3Coeffs[instanceNo][_c4]*workT3[r][2+buffer]);
}

//+-------------------------------------------------------------------
//|                                                                  
//+-------------------------------------------------------------------
//
//
//
//
//

void CleanPoint(int i,double& first[],double& second[])
{
   if ((second[i]  != EMPTY_VALUE) && (second[i+1] != EMPTY_VALUE))
        second[i+1] = EMPTY_VALUE;
   else
      if ((first[i] != EMPTY_VALUE) && (first[i+1] != EMPTY_VALUE) && (first[i+2] == EMPTY_VALUE))
          first[i+1] = EMPTY_VALUE;
}

//
//
//
//
//

void PlotPoint(int i,double& first[],double& second[],double& from[])
{
   if (first[i+1] == EMPTY_VALUE)
      {
         if (first[i+2] == EMPTY_VALUE) {
                first[i]   = from[i];
                first[i+1] = from[i+1];
                second[i]  = EMPTY_VALUE;
            }
         else {
                second[i]   =  from[i];
                second[i+1] =  from[i+1];
                first[i]    = EMPTY_VALUE;
            }
      }
   else
      {
         first[i]  = from[i];
         second[i] = EMPTY_VALUE;
      }
}

//-------------------------------------------------------------------
//
//-------------------------------------------------------------------
//
//
//
//
//

string sTfTable[] = {"M1","M5","M10","M15","M30","H1","H4","D1","W1","MN"};
int    iTfTable[] = {1,5,10,15,30,60,240,1440,10080,43200};

//
//
//
//
//

int stringToTimeFrame(string tfs)
{
   tfs = stringUpperCase(tfs);
   for (int i=ArraySize(iTfTable)-1; i>=0; i--)
         if (tfs==sTfTable[i] || tfs==""+iTfTable[i]) return(MathMax(iTfTable[i],Period()));
                                                      return(Period());
}
string timeFrameToString(int tf)
{
   for (int i=ArraySize(iTfTable)-1; i>=0; i--) 
         if (tf==iTfTable[i]) return(sTfTable[i]);
                              return("");
}

//
//
//
//
//

string stringUpperCase(string str)
{
   string   s = str;

   for (int length=StringLen(str)-1; length>=0; length--)
   {
      int tchar = StringGetChar(s, length);
         if((tchar > 96 && tchar < 123) || (tchar > 223 && tchar < 256))
                     s = StringSetChar(s, length, tchar - 32);
         else if(tchar > -33 && tchar < 0)
                     s = StringSetChar(s, length, tchar + 224);
   }
   return(s);
}